package formdef.plugin.example.employee.business;

import java.util.*;
import java.text.SimpleDateFormat;
import java.text.ParseException;
import java.sql.Timestamp;

import org.apache.commons.logging.LogFactory;
import org.apache.commons.logging.Log;

import formdef.plugin.example.employee.business.Employee;

public class EmployeeHandler {

    /** the business tier instance */
    private static EmployeeHandler instance = new EmployeeHandler();

    /** all available employees */
    private Map employees;

    /** the next available employee number */
    private int nextEmployeeNumber = 1;

    /** the logger instance */
    private static Log log = LogFactory.getLog(EmployeeHandler.class);

    /** hide the constructor */
    private EmployeeHandler() {
    }

    /** return the business tier instance */
    public static EmployeeHandler getInstance() {
        return instance;
    }

    /** get all employees, or create a fresh set if there isn't any */
    private Map getEmployees() {
        if ((employees == null) || (employees.size() <= 0)) {
            try {
                employees = createEmployees();
            } catch (ParseException e) {
                log.error("Unexpected exception while creating employees.", e);
                throw new RuntimeException(e);
            }
        }
        return employees;
    }

    /** get all employees as an array*/
    public Employee[] getEmployeeArray() {
        Map employees = getEmployees();
        ArrayList list = new ArrayList(employees.size());
        Iterator iter = employees.keySet().iterator();
        while (iter.hasNext()) {
            Employee employee = (Employee) employees.get(iter.next());
            list.add(employee);
        }
        return (Employee[]) list.toArray(new Employee[list.size()]);
    }

    /** create some employees to play with */
    private Map createEmployees()
            throws ParseException {
        if (log.isDebugEnabled()) {
            log.debug("Creating classes for testing...");
        }
        Map employees = new LinkedHashMap();

        SimpleDateFormat sdf = new SimpleDateFormat("MM/dd/yyyy");
        Employee employee = new Employee();
        employee.setNumber(getNextEmployeeNumber());
        employee.setName("John Doe");
        employee.setAddress("123 Main St.");
        employee.setBirth(sdf.parse("12/25/1970"));
        employee.setMode(Mode.FULL_TIME);
        employee.setSalary(125000);
        log.debug("Adding " + employee);
        employees.put(new Integer(employee.getNumber()),employee);

        employee = new Employee();
        employee.setNumber(getNextEmployeeNumber());
        employee.setName("Jane Doe");
        employee.setAddress("123 Main St.");
        employee.setBirth(sdf.parse("05/01/1978"));
        employee.setMode(Mode.PART_TIME);
        employee.setSalary(45000);
        log.debug("Adding " + employee);
        employees.put(new Integer(employee.getNumber()),employee);

        employee = new Employee();
        employee.setNumber(getNextEmployeeNumber());
        employee.setName("Mark Johnson");
        employee.setAddress("6 Maple Rd.");
        employee.setBirth(sdf.parse("08/19/1975"));
        employee.setMode(Mode.PART_TIME);
        employee.setSalary(20000);
        log.debug("Adding " + employee);
        employees.put(new Integer(employee.getNumber()),employee);

        employee = new Employee();
        employee.setNumber(getNextEmployeeNumber());
        employee.setName("Pam Rutherford");
        employee.setAddress("17 Oak Dr.");
        employee.setBirth(sdf.parse("02/13/1969"));
        employee.setMode(Mode.FULL_TIME);
        employee.setSalary(1200000);
        log.debug("Adding " + employee);
        employees.put(new Integer(employee.getNumber()),employee);

        return employees;
    }

    /** get the employee record with the given number */
    public Employee getEmployee(int employeeNumber) {
        return (Employee) getEmployees().get(new Integer(employeeNumber));
    }

    /** add the given employee to the list of existing ones */
    public void createEmployee(Employee employee) {
        employee.setNumber(getNextEmployeeNumber());
        getEmployees().put(new Integer(employee.getNumber()),employee);
    }

    /** update an existing employee with data from the given object */
    public void updateEmployee(Employee updates) {
        Employee employee = (Employee)
                getEmployees().get(new Integer(updates.getNumber()));

        if (employee == null) {
            throw new RuntimeException("Can't find the specified employee: "
                    + updates.getNumber());
        }
        employee.setName(updates.getName());
        employee.setAddress(updates.getAddress());
        employee.setSalary(updates.getSalary());
        employee.setBirth(updates.getBirth());
        employee.setMode(updates.getMode());
    }
    
    /** remove an existing employee from the list */
    public void removeEmployee(int employeeNumber) {
        employees.remove(new Integer(employeeNumber));
    }

    /** get the next available employee number */
    private int getNextEmployeeNumber() {
        //int result = 1;
        //Iterator iter = employees.keySet().iterator();
        //while (iter.hasNext()) {
        //    Employee employee = (Employee) employees.get(iter.next());
        //    if ((employee != null) && (employee.getNumber() >= result)) {
        //        result = employee.getNumber() + 1;
        //    }
        //}
        //return result;
        return nextEmployeeNumber++;
    }

}
