package formdef.plugin.example.employee.web;

import formdef.plugin.conversion.Converter;
import formdef.plugin.conversion.ConversionContext;
import formdef.plugin.example.employee.business.Mode;

/**
 * Converts data between Mode and String types.
 */
public class ModeConverter implements Converter {

    /**
     * <p>Converts the value to the specified type.</p>
     * 
     * @param context the {@link ConversionContext} holding information
     * about the value to convert and additional conversion parameters
     * @return the equivalent of the given value, in the specified type
     */
    public Object convert(ConversionContext context) {
        Class type = context.getType();
        Object value = context.getValue();
        
        if (type.equals(Mode.class)) {
            // the output should be of type Mode
            if ("F".equalsIgnoreCase((String)value)) {
                return Mode.FULL_TIME;
            } else {
                return Mode.PART_TIME;
            }
        } else if (type.equals(String.class)) {
            // the output should be a String
            if (Mode.FULL_TIME == value) {
                return "F";
            } else {
                return "P";
            }
        } else {
            throw new IllegalArgumentException("Unsupported type parameter: " + type);
        }
    }

}
