
package formdef.plugin.validator;

import java.util.StringTokenizer;
import java.io.BufferedInputStream;
import java.io.InputStream;
import java.io.IOException;
import javax.servlet.ServletException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.validator.ValidatorResources;
import org.apache.struts.action.ActionServlet;
import org.apache.struts.config.ModuleConfig;

import formdef.plugin.validator.digester.ValidatorResourcesInitializer;

/**
 * Overrides the implementation of {@link #initResources} to allow
 * a single XML to contain both form definitions and validation rules.
 */
public class CombinedValidatorPlugIn
        extends org.apache.struts.validator.ValidatorPlugIn {

    /**
     * Commons Logging instance.
     */
    private static Log log = LogFactory.getLog(CombinedValidatorPlugIn.class);

    /**
     * Delimitter for Validator resources.
     */
    private final static String RESOURCE_DELIM = ",";

    /**
     * The {@link ActionServlet} owning this application.
     */
    private ActionServlet servlet = null;


    /**
     * Overrides the implementation of init to gain access to the servlet
     * instance.
     *
     * @param servlet The ActionServlet for our application
     * @param config The ModuleConfig for our owning module
     *
     * @exception ServletException if we cannot configure ourselves correctly
     */
    public void init(ActionServlet servlet, ModuleConfig config)
            throws ServletException {
        this.servlet = servlet;
        super.init(servlet, config);
    }



    /**
     * Initialize the validator resources for this module.
     *
     * @exception IOException if an input/output error is encountered, not thrown by this implementation
     * @exception ServletException if we cannot initialize these resources, not thrown by this implementation
     */
    protected void initResources() throws IOException, ServletException {
        this.resources = new ValidatorResources();

        if (getPathnames() == null || getPathnames().length() <= 0) {
            return;
        }

        StringTokenizer st = new StringTokenizer(getPathnames(), RESOURCE_DELIM);

        while (st.hasMoreTokens()) {
            String validatorRules = st.nextToken().trim();

            if (log.isInfoEnabled()) {
                log.info(
                    "Loading validation rules file from '" + validatorRules + "'");
            }

            InputStream input =
                servlet.getServletContext().getResourceAsStream(validatorRules);

            if (input != null) {
                BufferedInputStream bis = new BufferedInputStream(input);

                try {
                    // pass in false so resources aren't processed
                    // until last file is loaded
                    ValidatorResourcesInitializer.initialize(resources, bis, false);
                } catch (IOException e) {
                    log.error(e.getMessage(), e);
                } finally {
                    bis.close();
                }

            } else {
                log.error(
                    "Skipping validation rules file from '"
                        + validatorRules
                        + "'.  No stream could be opened.");
            }
        }

        resources.process();

    }


}
